// The module observes all the requests coming from the VE optimize pages for
// the catching cases when page loading into the iframe is blocked due to the
// mixed active content error.
var mixedContentListener = (function() {
    var requests = [],
        navigations = [],
        IDs = [],
        frameReletions = [],
        isListened = false;

    var isObservedFrame = function(tabId, frameId) {
        if (tabManager.getWTTab(tabId) && IDs.indexOf(frameId) !== -1) {
            return true;
        }
        return false;
    };

    var deleteFromArray = function(tabId, frameId, array) {
        for (var i = 0, len = array.length; i < len; ++i) {
            if (array[i].tabId === tabId && array[i].frameId === frameId) {
                array.splice(i, 1);
                break;
            }
        }
    };

    var cleanRequestData = function(details) {
        deleteFromArray(details.tabId, details.frameId, requests);
        deleteFromArray(details.tabId, details.frameId, navigations);
    };

    var addRequest = function(details) {
        requests.push(details);
    };

    var beforeNavigate = function(details) {
        if (tabManager.getWTTab(details.tabId)) {
            IDs.push(details.frameId);
            frameReletions.push({
                frameId: details.frameId,
                parentFrameId: details.parentFrameId
            });
            navigations.push(details);
        }
    };

    var isJsAction = function(url) {
        return url.indexOf('mailto:') !== -1 || url.indexOf('javascript:') !== -1;
    };

    var getParentFrameID = function(frameId) {
        for (var i = 0, len = frameReletions.length; i < len; ++i) {
            if (frameReletions[i].frameId === frameId) {
                return frameReletions[i].parentFrameId;
            }
        }
        return null;
    };

    var sendMixedContentMessage = function(details) {
        chrome.tabs.sendMessage(details.tabId,
            {action: 'mixed-content', url: details.url},
            function(response) {}
        );
    };

    var handleMixedContentError = function(details) {
        var parentFrameId = getParentFrameID(details.frameId);
        if (parentFrameId === 0) {
            sendMixedContentMessage(details);
        }
        if (parentFrameId > 0) {
            chrome.webNavigation.getFrame(
                {tabId: details.tabId, frameId: parentFrameId},
                function(parentDetails) {
                    if (isURLInHosts(parentDetails.url, VE_ALLOWED_HOSTS)) {
                        sendMixedContentMessage(details);
                    }
                }
            );
        }
    };
    // The net::ERR_ABORTED error is present for the following cases:
    // 1. When navigation has been interrupted by user or js scripts
    // 2. When navigation has been interrupted by browser for blocking
    // the mixed active contents.
    var handleAbortedError = function(details) {
        var isMixedContent = true;
        for (var i = 0, len = requests.length; i < len; ++i) {
            if (requests[i].tabId === details.tabId &&
                requests[i].frameId === details.frameId) {
                isMixedContent = false;
                requests.splice(i, 1);
                break;
            }
        }

        if (isMixedContent && !isJsAction(details.url)) {
            handleMixedContentError(details);
        }

        deleteFromArray(details.tabId, details.frameId, navigations);
    };

    var errorOccurred = function(details) {
        if (isObservedFrame(details.tabId, details.frameId)) {
            if (details.error === 'net::ERR_ABORTED') {
                handleAbortedError(details);
            } else {
                cleanRequestData(details);
            }
        }
    };

    // In the chrome versions from 50 to 55 when navigation is blocked due to
    // the mixed active content then the navigation url is changed to
    // 'about:blank'
    var isMixedContent = function(details) {
        for (var i = 0, len = navigations.length; i < len; ++i) {
            if (navigations[i].tabId === details.tabId &&
                navigations[i].frameId === details.frameId &&
                navigations[i].url === 'about:blank') {
                return false;
            }
        }
        return true;
    };

    var completed = function(details) {
        if (isObservedFrame(details.tabId, details.frameId)) {
            // There are two cases when url gets about:blank value:
            // 1. When it's failed due to the mixed active content error
            // versions from 50 to 55)
            // 2. When an iframe is created without setting the src attribute
            // or it’s filled with “about:blank” value.
            // If it’s the second case the request details should be cleaned
            // from the data which should be processed in handleAbortedError
            // method for uniquely identifying mixed active content errors
            if (details.url !== 'about:blank' || !isMixedContent(details)) {
                cleanRequestData(details);
            }
        }
    };

    var addNavigationListeners = function() {
        var filter = {
           urls: ['*://*/*'],
           types: ['sub_frame']
        }, navigation = chrome.webNavigation;

        navigation.onBeforeNavigate.addListener(beforeNavigate, filter);

        navigation.onErrorOccurred.addListener(errorOccurred, filter);

        navigation.onCompleted.addListener(completed, filter);
    };

    var start = function(tabId) {
        var filter = {
            urls: ['*://*/*'],
            types: ['sub_frame'],
            tabId: tabId
        };

        chrome.webRequest.onBeforeSendHeaders.addListener(
            addRequest, filter, ['blocking', 'requestHeaders']
        );

        // 'start' function will be called for any browser tabs running VE
        // optimize url and since the webNavigation cannot be filtered by
        // the tabId we should keep one listener for all tabs.
        if (!isListened) {
            isListened = true;
            addNavigationListeners();
        }
    };

    var stop = function() {
        chrome.webRequest.onBeforeSendHeaders.removeListener(addRequest);
        chrome.webNavigation.onBeforeNavigate.removeListener(beforeNavigate);
        chrome.webNavigation.onErrorOccurred.removeListener(errorOccurred);
        chrome.webNavigation.onCompleted.removeListener(completed);
        requests = [];
        navigations = [];
        IDs = [];
        isListened = false;
    };

    return {
        start: start,
        stop: stop
    };
})();

optimizeListener.register(mixedContentListener);
